/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package gravsphcolldemo;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

import java.lang.IllegalArgumentException;
import java.lang.IllegalStateException;

import java.util.Vector;

import noope.output.MessageBox;

import noope.input.ArrayBlockReader;
import noope.input.BRLoadingException;
import noope.input.Valid;

import noope.core.Physics;

/**
 * Title:        AWT Gravity and Sphere Collision Demo Applet
 * Description:  This applet demonstrates the use of NOOPE to simulate the Gravity and SphereCollision laws. It uses AWT, so it can be run in old browsers.
 *
 * @author Miklos Reiter
 * @version May 2001
 */

public class Applet2 extends Applet {
    /**
     * List of the ProjectionBrData objects entered by the user and displayed in the List.
     */
    protected Vector projections = new Vector();

    /**
     * List of the EntityBrData objects entered by the user and displayed in the List.
     */
    protected Vector entities = new Vector();

    private int numberOfProjectionLaws;
    private String[] projectionLawNames;

  boolean isStandalone = false;
  private Frame frame; // stores either the frame containing the applet, if
  // it is run standalone, or a hidden frame
  Button buttonGo = new Button();
  BorderLayout borderLayout1 = new BorderLayout();
  Panel panelGlobalSettings = new Panel();
  Label lblG = new Label();
  TextField txtG = new TextField();
  Panel panelProjections = new Panel();
  List listProjections = new List();
  BorderLayout borderLayout2 = new BorderLayout();
  Panel panelProjButtons = new Panel();
  BorderLayout borderLayout3 = new BorderLayout();
  Button buttonEditProj = new Button();
  Button buttonNewProj = new Button();
  Button buttonDelProj = new Button();
  Label lblProjTitle = new Label();
  Panel panelEntities = new Panel();
  BorderLayout borderLayout4 = new BorderLayout();
  Label lblEntitiesTitle = new Label();
  List listEntities = new List();
  Panel entityButtonPanel = new Panel();
  Button buttonDelEntity = new Button();
  BorderLayout borderLayout5 = new BorderLayout();
  Button buttonNewEntity = new Button();
  Button buttonEditEntity = new Button();
  /**Get a parameter value*/
  public String getParameter(String key, String def) {
    return isStandalone ? System.getProperty(key, def) :
      (getParameter(key) != null ? getParameter(key) : def);
  }

  /**Construct the applet*/
  public Applet2() {
  }
  /**Initialize the applet*/
  public void init() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  /**Component initialization*/
  private void jbInit() throws Exception {
    // if not running standalone, create a hidden frame that can be used
    // as an owner for dialogs
    if (!isStandalone) frame = new Frame();
    buttonGo.setLabel("Go!");
    buttonGo.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonGo_actionPerformed(e);
      }
    });
    this.setLayout(borderLayout1);
    lblG.setText("G = ");
    txtG.setText("6.6732e-11");
    panelProjections.setLayout(borderLayout2);
    panelProjButtons.setLayout(borderLayout3);
    buttonEditProj.setLabel("Edit");
    buttonEditProj.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonEditProj_actionPerformed(e);
      }
    });
    buttonNewProj.setLabel("New");
    buttonNewProj.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonNewProj_actionPerformed(e);
      }
    });
    buttonDelProj.setLabel("Delete");
    buttonDelProj.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonDelProj_actionPerformed(e);
      }
    });
    lblProjTitle.setFont(new java.awt.Font("SansSerif", 1, 15));
    lblProjTitle.setText("Projections");
    panelEntities.setLayout(borderLayout4);
    lblEntitiesTitle.setFont(new java.awt.Font("SansSerif", 1, 15));
    lblEntitiesTitle.setText("Entities");
    buttonDelEntity.setLabel("Delete");
    buttonDelEntity.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonDelEntity_actionPerformed(e);
      }
    });
    entityButtonPanel.setLayout(borderLayout5);
    buttonNewEntity.setLabel("New");
    buttonNewEntity.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonNewEntity_actionPerformed(e);
      }
    });
    buttonEditEntity.setLabel("Edit");
    buttonEditEntity.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonEditEntity_actionPerformed(e);
      }
    });
    this.add(buttonGo, BorderLayout.SOUTH);
    this.add(panelGlobalSettings, BorderLayout.NORTH);
    panelGlobalSettings.add(lblG, null);
    panelGlobalSettings.add(txtG, null);
    this.add(panelProjections, BorderLayout.WEST);
    panelProjections.add(listProjections, BorderLayout.CENTER);
    panelProjections.add(panelProjButtons, BorderLayout.SOUTH);
    panelProjButtons.add(buttonEditProj, BorderLayout.CENTER);
    panelProjButtons.add(buttonNewProj, BorderLayout.WEST);
    panelProjButtons.add(buttonDelProj, BorderLayout.EAST);
    panelProjections.add(lblProjTitle, BorderLayout.NORTH);
    this.add(panelEntities, BorderLayout.CENTER);
    panelEntities.add(lblEntitiesTitle, BorderLayout.NORTH);
    panelEntities.add(listEntities, BorderLayout.CENTER);
    panelEntities.add(entityButtonPanel, BorderLayout.SOUTH);
    entityButtonPanel.add(buttonNewEntity, BorderLayout.WEST);
    entityButtonPanel.add(buttonDelEntity, BorderLayout.EAST);
    entityButtonPanel.add(buttonEditEntity, BorderLayout.CENTER);

    setupInitialLists();
  }

    /** Sets up initial entries in the Projection + Entities lists. */
    private void setupInitialLists() throws IllegalArgumentException
    {
	ProjectionBrData p1 = new ProjectionBrData
	    ("x-y plane", "(0,0,1)", "-4.0", "4.0", "-4.0", "4.0"),
	    p2 = new ProjectionBrData
		("x-z plane", "(0,-1,0)", "-4.0", "4.0", "-4.0", "4.0");

	projections.addElement(p1);
	listProjections.add(p1.toString());
	projections.addElement(p2);
	listProjections.add(p2.toString());

	EntityBrData e1 = new EntityBrData
	    ("Entity1", "1.0e10", "0.1", "(1,0,0)", "0", "red"),
	    e2 = new EntityBrData
		("Entity2", "2.0e10", "0.2", "(0,0,0)", "(0,0.5,0.25)", "blue"),
	    e3 = new EntityBrData
		("Entity3", "1e10", "0.2", "(-1,0,0)", "(0,0,0)", "green");

	entities.addElement(e1);
	listEntities.add(e1.toString());
	entities.addElement(e2);
	listEntities.add(e2.toString());
	entities.addElement(e3);
	listEntities.add(e3.toString());
    }

  /**Start the applet*/
  public void start() {
  }
  /**Stop the applet*/
  public void stop() {
  }
  /**Destroy the applet*/
  public void destroy() {
  }
  /**Get Applet information*/
  public String getAppletInfo() {
    return "Applet Information";
  }
  /**Get parameter info*/
  public String[][] getParameterInfo() {
    return null;
  }
  /**Main method*/
  public static void main(String[] args) {
    Applet2 applet = new Applet2();
    applet.isStandalone = true;
    applet.frame = new Frame() {
      protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
          System.exit(0);
        }
      }
      public synchronized void setTitle(String title) {
        super.setTitle(title);
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
      }
    };
    applet.frame.setTitle("Applet Frame");
    applet.frame.add(applet, BorderLayout.CENTER);
    applet.init();
    applet.start();
    applet.frame.setSize(400,320);
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    applet.frame.setLocation((d.width - applet.frame.getSize().width) / 2, (d.height - applet.frame.getSize().height) / 2);
    applet.frame.setVisible(true);
  }

    /**
     * Called when the user presses the Go button, starts the simulation.
     */
    protected void buttonGo_actionPerformed(ActionEvent e)
    {
	ArrayBlockReader physBr = null;
	Physics phys = null;
	try
	    {
		physBr = getPhysBr();
		phys = new Physics(physBr, new noope.input.HardCodedLawLoader(),
				   new noope.input.HardCodedEntityLoader(),
				   new noope.input.HardCodedLawPropertyRecordLoader());
	    }
	catch (IllegalStateException f) // not all data entered
	    {
		// just return, a message is displayed at the appropriate place
		return;
	    }
	catch (BRLoadingException f)
	    {
		MessageBox.displayMessage(frame, "Error", "Error occurred while constructing"
					  + " Physics. Message: " + f);
		f.printStackTrace();
		
		if (phys != null) phys.dispose(); // tidy up phys
		
		return;
	    }
	AWTRunDialog runD = new AWTRunDialog(frame);
	runD.setModal(true);
	runD.setTitle("Simulation");
	runD.setPhysics(phys);
	runD.startSimulation();
	runD.show();
    }


    /** returns a BlockReader object representing a Physics object. */
    private ArrayBlockReader getPhysBr() throws IllegalStateException
    {
	String header = "physics", parameter = null;
	String[][] entries = null;
	ArrayBlockReader[] subblocks = {getLawsBr(getLawBrs()), getEntitiesBr(getEntityBrs())};
	
	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    /** returns an array of BlockReader objects representing a Gravity, a SphereCollision and various
	OutputProjection laws. */
    private ArrayBlockReader[] getLawBrs() throws IllegalStateException
    {
	int i = 0; // counter variable for FOR-loop
	// get G
	String sG = txtG.getText();
	if (sG.equals("") || !Valid.validDouble(sG))
	{
	    MessageBox.displayMessage(frame, "Error", "Please enter a double for G.");
	    txtG.requestFocus();
	    throw new IllegalStateException();
	}
		
	int outLawN = projections.size();

	numberOfProjectionLaws = outLawN; // needed when constructing the Entity BlockReaders
	
	ArrayBlockReader[] result = new ArrayBlockReader[outLawN + 2];
	projectionLawNames = new String[outLawN]; // needed when constructing the Entity
	// BlockReaders

	result[0] = newGravBr(sG);
	result[1] = newSphereCollBr();

	// x and y coordinates of first 4 OutputProjection windows
	int nOfCoords = 4; // number of determined x/y coordinates
	String [] xarray = {"0", "300", "0", "300"};
	String [] yarray = {"0", "0", "300", "300"};
	
	try
	{
	    String x = null, y = null;
	    for (i = 2; i < outLawN + 2; i++)
	    {
		if ((i - 1) <= nOfCoords)
		{
		    x = xarray[i - 2];
		    y = yarray[i - 2];
		}
		else
		{
		    x = y = "0";
		}
		result[i] = ((ProjectionBrData)projections.elementAt(i - 2)).getArrayBlockReader(x, y);
		projectionLawNames[i - 2] =((ProjectionBrData)projections.elementAt(i - 2)).getName();
	    }
	}
	catch (IllegalArgumentException e)
	{
	    MessageBox.displayMessage(frame, "Error", "Could not create ArrayBlockReader from "
				      +"Projection " + i + ". This should not happen!");
	    e.printStackTrace();
	    throw new IllegalStateException();
	}

	return result;
    }

    /** returns an array of BlockReaders object representing the various entities. */
    private ArrayBlockReader[] getEntityBrs() throws IllegalStateException
    {
	int entityCount = entities.size();
	int i = 0; // counter variable for FOR loop
	
	ArrayBlockReader[] result = new ArrayBlockReader[entityCount];
	
	try
	{
	    for (i = 0; i < entityCount; i++)
		{
		    result[i] = ((EntityBrData)entities.elementAt(i)).getArrayBlockReader
			(projectionLawNames, numberOfProjectionLaws);
		}
	}
	catch (IllegalArgumentException e)
	{
	    MessageBox.displayMessage(frame, "Error", "Could not create ArrayBlockReader from "
				      +"Entity " + i + ". This should not happen!");
	    e.printStackTrace();
	    throw new IllegalStateException();
	}
	return result;
    }


    void buttonNewProj_actionPerformed(ActionEvent e) 
    {
	ProjectionBrData newData = EditProjectionDialog.newProjectionDialog(frame);
	if (newData != null)
	    {
		try
		    {
			listProjections.add(newData.toString());
			projections.addElement(newData);
		    }
		catch (IllegalArgumentException f)
		    {
			MessageBox.displayMessage(frame, "Error", "Exception in Applet2"
						  +".buttonNewProj_actionPerformed. "
						  +"This should not happen.");
			f.printStackTrace();
		    }
	    }
    }

    private ArrayBlockReader getEntitiesBr(ArrayBlockReader[] subblocks)
    {
	String header = "entities", parameter = null;
	String[][] entries = null;

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    // creates the laws {...} block containing all the laws as subblocks
    private ArrayBlockReader getLawsBr(ArrayBlockReader[] subblocks)
    {
	String header = "laws", parameter = null;
	String[][] entries = null;

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }    

    private ArrayBlockReader newGravBr(String sG) // creates an ArrayBlockReader defining a Gravity law
    {
	String header = "law", parameter = "Gravity"; 
	String[][] entries = 
	{{"name", "Gravity"},
	 {"G", sG}
	};
	ArrayBlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    /** Create a BlockReader object to construct a SphereCollision law from */
    private ArrayBlockReader newSphereCollBr()
    {
	String header = "law", parameter = "SphereCollision"; 
	String[][] entries = 
	{{"name", "SphereCollision"}
	};
	ArrayBlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }


  void buttonEditProj_actionPerformed(ActionEvent e) {
      if (listProjections.getItemCount() == 0)
	  {
	      MessageBox.displayMessage(frame, "Error", 
					"Please create projections with New first.");
	      return;
	  }

      int index = listProjections.getSelectedIndex();
      if (index == -1)
	  {
	      MessageBox.displayMessage(frame, "Error",
					"Please select an item.");
	      return;
	  }

      ProjectionBrData currentData = (ProjectionBrData)projections.elementAt(index);
      
      // give the user opportunity to change currentData
      currentData = EditProjectionDialog.editProjectionDialog(frame, currentData);
      
      // user hasn't cancelled the dialog
      if (currentData != null)
	  {
	      try
		  {
		      listProjections.replaceItem(currentData.toString(), index);
		      projections.setElementAt(currentData, index);
		  }
	      catch (IllegalArgumentException f)
		  {
		      MessageBox.displayMessage(frame, "Error",
						"Failed to format projection data as a string."
						+" This should not happen!");
		      f.printStackTrace();
		  }
	  }
  }

  void buttonDelProj_actionPerformed(ActionEvent e) {
      if (listProjections.getItemCount() == 0)
	  {
	      MessageBox.displayMessage(frame, "Error", 
					"Please create projections with New first.");
	      return;
	  }

      int index = listProjections.getSelectedIndex();
      if (index == -1)
	  {
	      MessageBox.displayMessage(frame, "Error",
					"Please select an item.");
	      return;
	  }
      listProjections.remove(index);
      projections.removeElementAt(index);
  }

    protected void buttonNewEntity_actionPerformed(ActionEvent e)
    {
	EntityBrData newData = EditEntityDialog.newEntityDialog(frame);
	if (newData != null)
	    {
		try
		    {
			listEntities.add(newData.toString());
			entities.addElement(newData);
		    }
		catch (IllegalArgumentException f)
		    {
			MessageBox.displayMessage(frame, "Error", "Exception in Applet2"
						  +".buttonNewProj_actionPerformed. "
						  +"This should not happen.");
			f.printStackTrace();
		    }
	    }
    }

    protected void buttonEditEntity_actionPerformed(ActionEvent e)
    {
	if (listEntities.getItemCount() == 0)
	    {
		MessageBox.displayMessage(frame, "Error", 
					  "Please create entities with New first.");
		return;
	    }
	
	int index = listEntities.getSelectedIndex();
	if (index == -1)
	    {
		MessageBox.displayMessage(frame, "Error",
					  "Please select an item.");
		return;
	    }
	
	EntityBrData currentData = (EntityBrData)entities.elementAt(index);
      
	// give the user opportunity to change currentData
	currentData = EditEntityDialog.editEntityDialog(frame, currentData);

	// user hasn't cancelled the dialog
	if (currentData != null)
	    {
		try
		    {
			listEntities.replaceItem(currentData.toString(), index);
			entities.setElementAt(currentData, index);
		    }
		catch (IllegalArgumentException f)
		    {
			MessageBox.displayMessage(frame, "Error",
						  "Failed to format entity data as a string."
						  +" This should not happen!");
			f.printStackTrace();
		    }
	    }	
    }
    
    protected void buttonDelEntity_actionPerformed(ActionEvent e)
    {
	if (listEntities.getItemCount() == 0)
	    {
		MessageBox.displayMessage(frame, "Error", 
					  "Please create entities with New first.");
		return;
	    }
	
	int index = listEntities.getSelectedIndex();
	if (index == -1)
	    {
		MessageBox.displayMessage(frame, "Error",
					  "Please select an item.");
		return;
	    }
	listEntities.remove(index);
	entities.removeElementAt(index);
    }
}
