/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

import noope.laws.Law;
import noope.core.Physics;

import java.lang.Class;
import java.lang.reflect.Constructor;
import java.lang.Object;

import java.lang.Integer;

/**
 *
 * This class implements a LawLoader that loads Law classes dynamically.
 *
 *
 */
public class DynamicLawLoader extends LawLoader
{
    /**
     * Stores the name of the default package to search in.
     */
    protected String defaultPackage;
    

    private Class[] paramsClasses;

    /**
     * Constructs a new DynamicLawLoader class that tries to load classes first with the exact
     * name given and then, if that fails, in the package specified.
     */
    public DynamicLawLoader(String aDefaultPackage) throws BRLoadingException
    {
	defaultPackage = aDefaultPackage;
	paramsClasses = new Class[]{BRClassLoaderUtils.getClassWithExactName
				    (null, "noope.input.BlockReader"),
				    java.lang.Integer.TYPE, 
				    BRClassLoaderUtils.getClassWithExactName
				    (null, "noope.core.Physics")};

    }

    /**
     * Tries to create a new instance of the law class with the given name.
     */
    public Law createLaw(String className, BlockReaderLocation brL, BlockReader br, 
				  int lawnumber, Physics phys)
	throws BRLoadingException
    {

	// Construct an instance of the class and add it to the Entities list
	Object[] params = {br, new Integer(lawnumber), phys};

	// Dynamically load the class
	Class c = BRClassLoaderUtils.getClassWithName(brL, className, defaultPackage);
	
	// assert it's a subclass of Law
	BRClassLoaderUtils.assertSubClass
	    (brL, BRClassLoaderUtils.getClassWithExactName(brL, "noope.laws.Law"), c);
		
	Constructor cons = BRClassLoaderUtils.getConstructor(brL, c, paramsClasses);
	
	return (Law)(BRClassLoaderUtils.getNewInstance(brL, c, cons, params));
    }
}
