/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

import noope.core.Physics;
import noope.entities.Entity;

/**
 * This is an implementation of EntityLoader that has all the Entity classes known to it
 * hard-coded.
 *
 * This is intended as a workaround for Netscape's paranoid SecurityManager and its use is
 * discouraged, except for Applets. If you are writing an application, use the
 * DynamicEntityLoader.
 *
 * @author Miklos Reiter
 * @version 28-Apr-2001
 *
 */
public class HardCodedEntityLoader extends EntityLoader
{
    /** Does nothing. */
    public HardCodedEntityLoader()
    {}

    /**
     * Creates a new Entity object. At the moment, only the Entity class is known to it.
     *
     * @arg className The name of the class to load.
     * @arg br This is the BlockReader to use to construct the instance.
     * @arg brL The BlockReaderLocation used to construct a BRLoadingException if the creation
     * of the instance fails.
     * @arg phys A reference to the Physics object running the simulation.
     */
    public Entity createEntity(String className, BlockReaderLocation brL, 
					BlockReader br, Physics phys)
	throws BRLoadingException
    {
	if (className.equals("Entity")) return new Entity(br, phys);
	if (className.equals("noope.entities.Entity")) return new Entity(br, phys);
	throw new BRLoadingException("The Entity class " + className + " is unknown to this "
				     + "HardCodedEntityLoader!", brL);
    }
}
