/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

import noope.entities.Entity;
import noope.laws.LawPropertyRecord;

/**
 * This is an implementation of the LawPropertyRecordLoader that has all the classes it can load
 * hard-coded in, so it just switches on the className it is given.
 *
 * This is a workaround for Netscape's paranoid SecurityManager! If you want something for use
 * in applications, use DynamicLawPropertyRecordLoader.
 *
 * @author Miklos Reiter
 * @version 29-Apr-2001
 *
 */
public class HardCodedLawPropertyRecordLoader extends LawPropertyRecordLoader
{
    /**
     * This constructor does nothing.
     */
    public HardCodedLawPropertyRecordLoader()
    {}

    /**
     * Creates a new instance of the given class. This version only supports the subclasses of
     * OutputProjectionPropertyRecord, as of 29-Apr-2001.
     *
     * If subclassed, override this method and in the modified version, first check for new
     * classes added and then call the inherited method.
     *
     * @arg className The name of the class to load.
     * @arg br This is the BlockReader to use to construct the instance.
     * @arg brL The BlockReaderLocation used to construct a BRLoadingException if the creation
     * of the instance fails.
     * @arg ent The Entity for which the LawPropertyRecord is going to be constructed.
     */
    public LawPropertyRecord createLawPropertyRecord(String className, 
						     BlockReaderLocation brL, 
						     BlockReader br, Entity ent)
	throws BRLoadingException
    {
	if (className.equals("OutputProjectionCirclePropertyRecord"))
	    return new noope.output.OutputProjectionCirclePropertyRecord(br, ent);
	if (className.equals("OutputProjectionPointPropertyRecord"))
	    return new noope.output.OutputProjectionPointPropertyRecord(br, ent);

	// with package prefix
	if (className.equals("noope.output.OutputProjectionCirclePropertyRecord"))
	    return new noope.output.OutputProjectionCirclePropertyRecord(br, ent);
	if (className.equals("noope.output.OutputProjectionPointPropertyRecord"))
	    return new noope.output.OutputProjectionPointPropertyRecord(br, ent);

	// if we got so far and haven't found anything, throw an exception
	throw new BRLoadingException("LawPropertyRecord class " + 
             className + " is unknown to this HardCodedLawPropertyRecordLoader!", brL);
    }
}
