/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;

import noope.core.Vector3D;

import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;

import noope.entities.Entity;

import java.text.ParseException;

import java.awt.Color;
import java.awt.Graphics2D;

import java.lang.Double;

/** This is a LawPropertyRecord class associated with the OutputProjection law. It is the one
    of which an instance is returned by OutputProjection.createNewPropertyRecord by default, that
    is, if no class name is given as the parameter defining the LawPropertyRecord. It represents
    the object as a small square whose sidelength is given by pointSize (that is initialized to the
    value defaultPointSize if nothing else is given.
    @author Miklos Reiter
    @version 29 March 2001
 */
public class OutputProjectionPointPropertyRecord extends OutputProjectionColoredPropertyRecord
{
    /** The size of a point. */
    protected double pointSize;

    /** The default pointSize. */
    public static double defaultPointSize = 0.1;

    /** Constructs a new OutputProjectionPointPropertyRecord object from the BlockReader br and
	for the Entity ent. The BlockReader can be empty or even null, in which case
	defaultColor is used. In addition to the fields loaded from br by the inherited constructor,
	it tries to load the double "pointSize", if it fails, it uses
	defaultPointSize. Caution: if pointSize is too small, the Entity won't appear at all!
    */
    public OutputProjectionPointPropertyRecord(BlockReader br, Entity ent) throws BRLoadingException
    {
	super(br, ent);

	String[] brEntry; // for reading entries from br
	
	// try to load pointSize now (optional so use peek...):
	if (br != null && br.hasMoreEntries() && ((brEntry = br.peekNextEntry())[0].equals("pointSize")))
	    {
		try
		    {
			br.getNextEntry(); // advance pointer
			pointSize = Double.valueOf(brEntry[1]).doubleValue();
		    }
		catch (NumberFormatException e)
		    {
			throw (new
			    BRLoadingParseException("Failed to parse for double" , 
						    br.getEntryLocation()));
		    }
	    }
	else // don't complain if nothing is given, use default!
	    {
		pointSize = defaultPointSize;
	    }
    }


    /** This method draws a square of size pointSize on the Graphics object at the position
	(X,Y). */
    public void draw(Graphics2D g, double X, double Y)
    {
	double p2 = pointSize/2;
	
	// display new point
	g.setColor(color);
	g.fill(new java.awt.geom.Rectangle2D.Double(X-p2, Y-p2, pointSize, pointSize));
    }
}
