/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;

import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;

import noope.entities.Entity;

import noope.laws.LawPropertyRecord;

import java.awt.Graphics2D;


/** This is the LawPropertyRecord class associated with the OutputProjection law.
    @author Miklos Reiter
    @version 19 April 2001, fixed bug: paint should just call draw and not check whether coordinates
    have changed, since the screen is cleared before each screenshot now.
 */
abstract public class OutputProjectionPropertyRecord extends LawPropertyRecord
{
    protected double oldX = java.lang.Double.NaN, oldY = java.lang.Double.NaN;

    /** Constructs a new OutputProjectionPropertyRecord object from the BlockReader br and
	for the Entity ent. This base class does not load anything from br yet, it just calls
	the inherited constructor.
    */
    public OutputProjectionPropertyRecord(BlockReader br, Entity ent) throws BRLoadingException
    {
	super(br, ent);
    }


    /** This method just calls draw.
     */
    public void paint(Graphics2D g, double X, double Y)
    {
	draw(g, X, Y);
	oldX = X;
	oldY = Y;
    }
    
    /** This method draws the object on the Graphics object at the position (X,Y). This may
	be overridden for subclasses, to change drawing behaviour. */
    abstract public void draw(Graphics2D g, double X, double Y);
}
