/**
 * Copyright (C) 2001  James Kermode (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package test;

/* TestPhysics - James Kermode 22-Mar-2001
 * modified from TestAll_miklos, Miklos Reiter, March 21 2001
 *
 * Big test program to test the Physics, Laws, Entities, Law, LawPropertyRecord, 
 * Gravity, GravityPropertyRecord, Entity classes
 */

import java.util.ListIterator;

import noope.core.*;
import noope.input.*;
import noope.laws.Law;
import noope.entities.Entity;

public class TestPhysics
{
    public static void main(String[] argv) throws BRLoadingException
    {
	// Set up an ArrayBlockReader to load physics from
	BlockReader[] lawsBlocks = {newGravBr()};
        BlockReader[] entitiesBlocks = {newEntity1Br(), newEntity2Br()};
	BlockReader lawsBr = newLawsBr(lawsBlocks);
	BlockReader entitiesBr = newEntitiesBr(entitiesBlocks);
	BlockReader physicsBr = newPhysicsBr(lawsBr, entitiesBr);

	// Construct physics from this ArrayBlockReader
	Physics physics = new Physics(physicsBr);

	// Have a look at what's in physics
	printLaws(physics);
	printEntities(physics);

	// Run simulation for 0.1 seconds
	physics.step(0.1);

	// See what's changed
	printEntities(physics);
    }

    private static void printLaws(Physics physics)
    {
	System.out.println("Laws:\n-----");
	Law law;
	for (ListIterator i = physics.getLaws().listIterator(); i.hasNext(); )
	{
	    law = (Law)i.next();
		System.out.println(law+"\n");
	}
    }

    private static void printEntities(Physics physics)
    {
        System.out.println("\nEntities:\n---------");
	Entity entity;
	for (ListIterator i = physics.getEntities().listIterator(); i.hasNext(); )
	{
	    entity = (Entity)i.next();
		System.out.println(entity+"\n");
	}
    }
    
    // *** Miklos' ArrayBlockReader test functions:
	
    private static BlockReader newGravBr() // creates an ArrayBlockReader defining a Gravity law
    {
	String header = "law", parameter = "Gravity"; 
	// these aren't really needed, but every BlockReader has to have
	// them
	String[][] entries = 
	{{"name", "Gravity"}
	};
	BlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    private static BlockReader newEntity1Br() // creates an ArrayBlockReader defining an Entity
    {
	String header = "entity", parameter = ""; // these aren't really needed, but every BlockReader has to have
	// them
	String[][] entries = 
	{{"name", "Entity1"},
	 {"position", "(1, 0, 0)"},
	 {"velocity", "(0,1,0)"},
	 {"inertialMass", "1"}
	};
	BlockReader[] subblocks = {}; // don't give it a GravityPropertyRecord

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    private static BlockReader newEntity2Br() // creates an ArrayBlockReader defining an Entity
    {
	String header = "entity", parameter = ""; // these aren't really needed, but every BlockReader has to have
	// them
	String[][] entries = 
	{{"name", "Entity2"},
	 {"position", "(0, 0, 0)"},
	 {"velocity", "0"},
	 {"inertialMass", "2"}
	};
	BlockReader[] subblocks = {newGravPropBr()}; // give it a GravityPropertyRecord

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    private static BlockReader newGravPropBr() // creates an ArrayBlockReader defining a GravityPropertyRecord
    {
	String header = "Gravity", parameter = null; 
	String[][] entries = {
	{"gravitationalMass", "3"}
	};
	BlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    // *** ArrayBlockReader functions added by James Kermode
	
    // creates the entities {...} block containing all the entities as subblocks
    private static BlockReader newEntitiesBr(BlockReader[] subblocks)
    {
	String header = "entities", parameter = null;
	String[][] entries = null;

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }    

    // creates the laws {...} block containing all the laws as subblocks
    private static BlockReader newLawsBr(BlockReader[] subblocks)
    {
	String header = "laws", parameter = null;
	String[][] entries = null;

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }    

    // creates the physics {...} block containing laws {...} and entities{...} as subblocks
    private static BlockReader newPhysicsBr(BlockReader laws, BlockReader entities)
    {
	String header = "physics", parameter = null;
	String[][] entries = null;
	BlockReader[] subblocks = {laws, entities};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }    
}
